'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = base;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _isNodeFn = require('is-node-fn');

var _isNodeFn2 = _interopRequireDefault(_isNodeFn);

var _debug = require('../util/debug');

var _debug2 = _interopRequireDefault(_debug);

var _actions = require('../redux/actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Base connect component.
 * @param {Function} target
 * @param {String} dataKey
 * @param {*} fallbackDataValue
 * @returns {KasiaConnectedComponent}
 */
function base(target, dataKey, fallbackDataValue) {
  var _class, _temp;

  var displayName = target.displayName || target.name;

  return _temp = _class = function (_React$PureComponent) {
    _inherits(KasiaConnectedComponent, _React$PureComponent);

    function KasiaConnectedComponent() {
      _classCallCheck(this, KasiaConnectedComponent);

      return _possibleConstructorReturn(this, _React$PureComponent.apply(this, arguments));
    }

    /** Make request for new data from WP-API. */
    KasiaConnectedComponent.prototype._requestWpData = function _requestWpData(props) {
      var action = this._getRequestWpDataAction(props);
      this.props.dispatch(action);
    };

    /** Find the query for this component and its corresponding data
     *  and return props object containing them. */


    KasiaConnectedComponent.prototype._reconcileWpData = function _reconcileWpData(props, query) {
      var data = fallbackDataValue;

      if (query) {
        if (query.complete && query.OK) {
          if (query.preserve) {
            data = query.result;
          } else {
            data = this._makePropsData(props, query);
          }
        } else if (query.error) {
          console.log('[kasia] error in query for ' + displayName + ':');
          console.log(query.error);
        }
      }

      (0, _debug2.default)(displayName + ' has content on `props.kasia.' + dataKey + '`');

      return _defineProperty({
        query: query || {
          complete: false,
          OK: null
        }
      }, dataKey, data);
    };

    KasiaConnectedComponent.prototype.componentWillMount = function componentWillMount() {
      var state = this.props.wordpress;
      var queryId = this.queryId = state.__nextQueryId;
      var query = state.queries[queryId];

      (0, _debug2.default)(displayName + ' will mount with queryId=' + queryId);

      if (query && query.prepared) {
        (0, _debug2.default)(displayName + ' has prepared data at queryId=' + queryId);
        this.props.dispatch((0, _actions.incrementNextQueryId)());
      } else if (!(0, _isNodeFn2.default)()) {
        (0, _debug2.default)(displayName + ' initiating request in componentWillMount');
        this._requestWpData(this.props);
      }
    };

    KasiaConnectedComponent.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
      var willUpdate = this._shouldUpdate(this.props, nextProps, this.context.store.getState());
      if (willUpdate) {
        this.queryId = this.props.wordpress.__nextQueryId;
        (0, _debug2.default)(displayName + ' initiating request: queryId=' + this.queryId + ', props: ' + nextProps);
        this._requestWpData(nextProps);
      }
    };

    KasiaConnectedComponent.prototype.render = function render() {
      var query = this.props.wordpress.queries[this.queryId];
      var kasia = this._reconcileWpData(this.props, query);
      var props = Object.assign({}, this.props, { kasia: kasia });
      return _react2.default.createElement(target, props);
    };

    return KasiaConnectedComponent;
  }(_react2.default.PureComponent), _class.__kasia__ = true, _class.WrappedComponent = target, _class.contextTypes = {
    store: _propTypes2.default.object.isRequired }, _temp;
}