'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _lodash = require('lodash.get');

var _lodash2 = _interopRequireDefault(_lodash);

var _debug = require('../util/debug');

var _debug2 = _interopRequireDefault(_debug);

var _invariants = require('../invariants');

var _invariants2 = _interopRequireDefault(_invariants);

var _findEntities = require('../util/find-entities');

var _findEntities2 = _interopRequireDefault(_findEntities);

var _base2 = require('./base');

var _base3 = _interopRequireDefault(_base2);

var _util = require('./util');

var _actions = require('../redux/actions');

var _sagas = require('../redux/sagas');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

/**
 * Connect a component to arbitrary data from WordPress.
 *
 * The component will request new data via the given `queryFn` if `shouldUpdate` returns true.
 *
 * @example Get all posts by an author:
 * ```js
 * connectWpQuery((wpapi) => wpapi.posts().embed().author('David Bowie'), () => true)
 * ```
 *
 * @example Get all pages:
 * ```js
 * connectWpQuery((wpapi) => wpapi.pages().embed(), () => true)
 * ```
 *
 * @example Get custom content type by slug (content type registered at init):
 * ```js
 * connectWpQuery(
 *   (wpapi) => wpapi.news().slug('gullible-removed-from-the-dictionary').embed(),
 *   () => true
 * )
 * ```
 *
 * @example Update only when `props.id` changes:
 * ```js
 * connectWpQuery((wpapi, props) => {
 *   return wpapi.page().id(props.identifier()).embed().get()
 * }, 'id')
 * ```
 *
 * @param {Function} queryFn Function that returns a wpapi query
 * @param {Function|String|Object} [shouldUpdate] Inspect props to determine if new data request is made
 * @param {Object} [opts] Options object
 * @param {Function|String|Object} [opts.shouldUpdate] See `shouldUpdate` docs
 * @param {Boolean} [opts.preserve] Preserve result of query and pass this untouched to component (no normalisation)
 * @returns {Function} Decorated component
 */
function connectWpQuery(queryFn, _shouldUpdate2) {
  var opts = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  if ((typeof _shouldUpdate2 === 'undefined' ? 'undefined' : _typeof(_shouldUpdate2)) === 'object') {
    opts = _shouldUpdate2;
    _shouldUpdate2 = opts.shouldUpdate;
  }

  _invariants2.default.isObject('opts', opts);
  _invariants2.default.isFunction('queryFn', queryFn);

  if (_shouldUpdate2) {
    _invariants2.default.isShouldUpdate(_shouldUpdate2);
  }

  if (typeof _shouldUpdate2 === 'string') {
    _shouldUpdate2 = function shouldUpdate(thisProps, nextProps) {
      return (0, _lodash2.default)(thisProps, _shouldUpdate2) !== (0, _lodash2.default)(nextProps, _shouldUpdate2);
    };
  }

  return function (target) {
    var displayName = target.displayName || target.name;

    _invariants2.default.isNotWrapped(target, displayName);

    var KasiaConnectWpQueryComponent = function (_base) {
      _inherits(KasiaConnectWpQueryComponent, _base);

      function KasiaConnectWpQueryComponent() {
        _classCallCheck(this, KasiaConnectWpQueryComponent);

        return _possibleConstructorReturn(this, _base.apply(this, arguments));
      }

      KasiaConnectWpQueryComponent.preload = function preload(props, state) {
        (0, _debug2.default)(displayName, 'connectWpQuery preload with props:', props, 'state:', state);
        var wrappedQueryFn = (0, _util.wrapQueryFn)(queryFn, props, state);
        var action = (0, _actions.createQueryRequest)(wrappedQueryFn, opts.preserve);
        return [_sagas.fetch, action];
      };

      KasiaConnectWpQueryComponent.prototype._getRequestWpDataAction = function _getRequestWpDataAction(props) {
        (0, _debug2.default)(displayName, 'connectWpQuery request with props:', props);
        var wrappedQueryFn = (0, _util.wrapQueryFn)(queryFn, props, this.context.store.getState());
        return (0, _actions.createQueryRequest)(wrappedQueryFn, opts.preserve);
      };

      KasiaConnectWpQueryComponent.prototype._makePropsData = function _makePropsData(_, query) {
        var state = this.props.wordpress;
        return (0, _findEntities2.default)(state.entities, query.entities, state.__keyEntitiesBy);
      };

      KasiaConnectWpQueryComponent.prototype._shouldUpdate = function _shouldUpdate() {
        if (_shouldUpdate2) {
          return _shouldUpdate2.apply(undefined, arguments);
        } else {
          return false;
        }
      };

      return KasiaConnectWpQueryComponent;
    }((0, _base3.default)(target, 'data', {}));

    return (0, _util.connect)(KasiaConnectWpQueryComponent);
  };
}
exports.default = connectWpQuery;