'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _preload = require('./util/preload');

Object.keys(_preload).forEach(function (key) {
  if (key === "default" || key === "__esModule") return;
  Object.defineProperty(exports, key, {
    enumerable: true,
    get: function get() {
      return _preload[key];
    }
  });
});
exports.runSagas = runSagas;

var _effects = require('redux-saga/effects');

var effects = _interopRequireWildcard(_effects);

var _debug2 = require('./util/debug');

var _debug3 = _interopRequireDefault(_debug2);

var _reducer = require('./redux/reducer');

var _reducer2 = _interopRequireDefault(_reducer);

var _invariants = require('./invariants');

var _invariants2 = _interopRequireDefault(_invariants);

var _contentTypesManager = require('./util/content-types-manager');

var _contentTypesManager2 = _interopRequireDefault(_contentTypesManager);

var _runSagas2 = require('./util/run-sagas');

var _runSagas3 = _interopRequireDefault(_runSagas2);

var _wpapi = require('./wpapi');

var _sagas = require('./redux/sagas');

var _actions = require('./redux/actions');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

exports.default = kasia;

// Components of the toolset that are extensible via plugins

var COMPONENTS_BASE = {
  sagas: [_sagas.watchRequests],
  reducers: {}

  /** Reset the connect internal query counter.
   *  Should be called before each SSR. */
};kasia.rewind = function (store) {
  store.dispatch((0, _actions.rewind)());
};

/** Run all `sagas` until they are complete. */
function runSagas(store, sagas) {
  kasia.rewind(store); // begin with a rewind
  sagas.push(function () {
    return (/*#__PURE__*/regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                // and end with a rewind
                store.dispatch((0, _actions.rewind)());

              case 1:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      })
    );
  });
  return (0, _runSagas3.default)(store, sagas);
}

/**
 * Configure Kasia.
 * @param {WP} opts.wpapi Instance of node-wpapi
 * @param {String} [opts.keyEntitiesBy] Property used to key entities in the store
 * @param {Boolean} [opts.debug] Log debug statements
 * @param {Array} [opts.plugins] Kasia plugins
 * @param {Array} [opts.contentTypes] Custom content type definition objects
 * @returns {Object} Kasia reducer
 */
function kasia() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var WP = opts.WP,
      wpapi = opts.wpapi,
      _opts$debug = opts.debug,
      _debug = _opts$debug === undefined ? false : _opts$debug,
      _opts$keyEntitiesBy = opts.keyEntitiesBy,
      keyEntitiesBy = _opts$keyEntitiesBy === undefined ? 'id' : _opts$keyEntitiesBy,
      _opts$plugins = opts.plugins,
      userPlugins = _opts$plugins === undefined ? [] : _opts$plugins,
      _opts$contentTypes = opts.contentTypes,
      contentTypes = _opts$contentTypes === undefined ? [] : _opts$contentTypes;

  (0, _debug2.toggleDebug)(_debug);

  (0, _debug3.default)('initialised with: ', opts);

  if (WP) {
    console.log('[kasia] config option `WP` is replaced by `wpapi` in v4.');
    wpapi = WP;
  }

  _invariants2.default.isWpApiInstance((0, _wpapi.setWP)(wpapi));
  _invariants2.default.isKeyEntitiesByOption(keyEntitiesBy);
  _invariants2.default.isArray('plugins', userPlugins);
  _invariants2.default.isArray('contentTypes', contentTypes);

  contentTypes.forEach(function (type) {
    return _contentTypesManager2.default.register(type);
  });

  // Merge plugins into internal sagas array and reducers object

  var _userPlugins$reduce = userPlugins.reduce(function (components, p, i) {
    var _components$sagas;

    var isArr = p instanceof Array;
    _invariants2.default.isPlugin('plugin at index ' + i, isArr ? p[0] : p);

    var _ref = isArr ? p[0](wpapi, p[1] || {}, opts) : p(wpapi, {}, opts),
        sagas = _ref.sagas,
        reducers = _ref.reducers;

    (_components$sagas = components.sagas).push.apply(_components$sagas, _toConsumableArray(sagas));
    Object.assign(components.reducers, reducers);
    return components;
  }, COMPONENTS_BASE),
      sagas = _userPlugins$reduce.sagas,
      reducers = _userPlugins$reduce.reducers;

  return {
    kasiaReducer: (0, _reducer2.default)({ keyEntitiesBy: keyEntitiesBy, reducers: reducers }),
    kasiaSagas: sagas.map(function (saga) {
      return effects.spawn(saga);
    }),
    kasiaActions: { createQueryRequest: _actions.createQueryRequest, createPostRequest: _actions.createPostRequest }
  };
}