'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var NODE_WPAPI_GITHUB_URL = 'http://bit.ly/2adfKKg';
var KASIA_URL = 'http://kasia.io';

function invariant(predicate, message) {
  if (!predicate) {
    for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
      args[_key - 2] = arguments[_key];
    }

    var interpolated = args.reduce(function (str, arg) {
      return str.replace(/%s/, arg);
    }, message);
    var err = new Error('[kasia] ' + interpolated);
    err.framesToPop = 1;
    throw err;
  }
}

exports.default = {
  isObject: function isObject(name, value) {
    return invariant((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object', 'Expecting %s to be object, got "%s".', name, typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isString: function isString(name, value) {
    return invariant(typeof value === 'string', 'Expecting %s to be string, got "%s".', name, typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isFunction: function isFunction(name, value) {
    return invariant(typeof value === 'function', 'Expecting %s to be function, got "%s".', name, typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isPlugin: function isPlugin(name, value) {
    return invariant(typeof value === 'function', 'Expecting %s to be function, got "%s". ' + 'Please file an issue with the plugin if you ' + 'think there might be a problem with it.', name, typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isArray: function isArray(name, value) {
    return invariant(Array.isArray(value), 'Expecting %s to be array, got "%s".', name, typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isBoolean: function isBoolean(name, value) {
    return invariant((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object', 'Expecting %s to be boolean, got "%s".', name, typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isShouldUpdate: function isShouldUpdate(value) {
    return invariant(typeof value === 'function' || typeof value === 'string' && value.length, 'Expecting shouldUpdate to be function or non-empty string, got "%s".', typeof value === 'undefined' ? 'undefined' : _typeof(value));
  },
  isWpApiInstance: function isWpApiInstance() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return invariant(typeof value.registerRoute === 'function', 'Expecting WP to be instance of `node-wpapi`. ' + ('See documentation: ' + NODE_WPAPI_GITHUB_URL + '.'));
  },
  isIdentifierArg: function isIdentifierArg(identifier) {
    return invariant(['function', 'string', 'number'].indexOf(typeof identifier === 'undefined' ? 'undefined' : _typeof(identifier)) > -1, 'Expecting id given to connectWpPost to be function/string/number, got "%s".', typeof identifier === 'undefined' ? 'undefined' : _typeof(identifier));
  },
  isValidContentTypeObject: function isValidContentTypeObject(obj) {
    return invariant(typeof obj.name === 'string' && typeof obj.plural === 'string' && typeof obj.slug === 'string', 'Invalid content type object. ' + ('See documentation: ' + KASIA_URL + '.'));
  },
  isValidContentType: function isValidContentType(contentTypeOptions, name, checkStr) {
    return invariant(typeof contentTypeOptions !== 'undefined', 'Content type "%s" is not recognised. ' + 'Pass built-ins from `kasia/types`, e.g. `connectWpPost(Post, ...)`. ' + 'Pass the name of custom content types, e.g. `connectWpPost("Book", ...)`. ' + 'Check %s.', name, checkStr);
  },
  isNewContentType: function isNewContentType(typesMap, contentType) {
    return invariant(typesMap && !typesMap.get(contentType.name), 'Content type with name "%s" already exists.', contentType.name);
  },
  isNotWrapped: function isNotWrapped(target, displayName) {
    return invariant(!target.__kasia, '%s is already wrapped by Kasia.', displayName);
  },
  isIdentifierValue: function isIdentifierValue(id, displayName) {
    return invariant(typeof id === 'string' || typeof id === 'number', 'The final identifier for ' + displayName + ' is invalid. ' + 'Expecting a string or number, got "%s".', typeof id === 'undefined' ? 'undefined' : _typeof(id));
  },
  hasWordpressObject: function hasWordpressObject(wordpress) {
    return invariant(wordpress, 'No `wordpress` object on the store. ' + 'Is your store configured correctly? ' + ('See documentation ' + KASIA_URL + '.'), typeof wordpress === 'undefined' ? 'undefined' : _typeof(wordpress));
  },
  isKeyEntitiesByOption: function isKeyEntitiesByOption(keyEntitiesBy) {
    return invariant(keyEntitiesBy === 'slug' || keyEntitiesBy === 'id', 'Expecting keyEntitiesBy to be "slug" or "id", got "%s".', keyEntitiesBy);
  }
};