'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.rewind = exports.incrementNextQueryId = exports.failRequest = exports.completeRequest = exports.acknowledgeRequest = exports.createQueryRequest = exports.createPostRequest = undefined;

var _constants = require('../constants');

/** Initiate a request for a single entity from the WP-API. */
var createPostRequest = exports.createPostRequest = function createPostRequest(contentType, identifier) {
  return {
    type: _constants.ActionTypes.RequestCreatePost,
    request: {
      contentType: contentType,
      identifier: identifier
    }
  };
};

/** Initiate an arbitrary request to the WP-API. */
var createQueryRequest = exports.createQueryRequest = function createQueryRequest(queryFn) {
  var preserve = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  return {
    type: _constants.ActionTypes.RequestCreateQuery,
    request: {
      queryFn: queryFn,
      preserve: preserve
    }
  };
};

/** Acknowledge a create* request by placing record of it on the store. */
var acknowledgeRequest = exports.acknowledgeRequest = function acknowledgeRequest(request) {
  return {
    type: _constants.ActionTypes.RequestAck,
    request: request
  };
};

/** Place the result of a successful request on the store */
var completeRequest = exports.completeRequest = function completeRequest(id, result) {
  return {
    type: _constants.ActionTypes.RequestComplete,
    request: {
      id: id,
      result: result
    }
  };
};

/** Update the record of a request with the error returned from a failed response. */
var failRequest = exports.failRequest = function failRequest(id, error) {
  return {
    type: _constants.ActionTypes.RequestFail,
    request: {
      id: id,
      error: error
    }
  };
};

/** Increment the next query ID. **/
var incrementNextQueryId = exports.incrementNextQueryId = function incrementNextQueryId() {
  return {
    type: _constants.ActionTypes.IncrementNextQueryId
  };
};

/** Reset the queryId counter to zero. */
var rewind = exports.rewind = function rewind() {
  return {
    type: _constants.ActionTypes.RewindQueryCounter
  };
};