'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.INITIAL_STATE = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.acknowledgeReducer = acknowledgeReducer;
exports.completeReducer = completeReducer;
exports.failReducer = failReducer;
exports.rewindReducer = rewindReducer;
exports.incrementNextQueryIdReducer = incrementNextQueryIdReducer;
exports.default = createReducer;

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _isNodeFn = require('is-node-fn');

var _isNodeFn2 = _interopRequireDefault(_isNodeFn);

var _pickEntityIds = require('../util/pick-entity-ids');

var _pickEntityIds2 = _interopRequireDefault(_pickEntityIds);

var _normalise = require('../util/normalise');

var _normalise2 = _interopRequireDefault(_normalise);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var INITIAL_STATE = exports.INITIAL_STATE = {
  __nextQueryId: 0,
  __keyEntitiesBy: 'id',
  // WP-API request/response metadata are stored here
  queries: {},
  // Entities are normalised and stored here
  entities: {}

  /**
   * Merge all native and plugin reducers such that a single function reduces for a single action type.
   * @param {Object} reducers Plugin reducers
   * @param {Function} normaliser Function to normalise response data
   * @returns {Object} Reducer object
   */
};function mergeNativeAndThirdPartyReducers(reducers, normaliser) {
  var _baseReducer;

  var baseReducer = (_baseReducer = {}, _defineProperty(_baseReducer, _constants.ActionTypes.RequestAck, [acknowledgeReducer]), _defineProperty(_baseReducer, _constants.ActionTypes.RequestComplete, [completeReducer(normaliser)]), _defineProperty(_baseReducer, _constants.ActionTypes.RequestFail, [failReducer]), _defineProperty(_baseReducer, _constants.ActionTypes.RewindQueryCounter, [rewindReducer]), _defineProperty(_baseReducer, _constants.ActionTypes.IncrementNextQueryId, [incrementNextQueryIdReducer]), _baseReducer);

  // Group reducers by their action type
  var reducersByActionType = Object.keys(reducers).reduce(function groupByActionType(reducer, actionType) {
    reducer[actionType] = [].concat(reducer[actionType] || [], reducers[actionType] || []);
    return reducer;
  }, baseReducer);

  var reducer = {};

  // Produce a single function for each action type

  var _loop = function _loop(actionType) {
    if (reducersByActionType[actionType].length > 1) {
      // Call each reducer function in succession, passing the state returned from each to the next
      reducer[actionType] = function (state, action) {
        return reducersByActionType[actionType].reduce(function (state, fn) {
          return fn(state, action);
        }, state);
      };
    } else {
      // Take the first and only function as the whole reducer
      reducer[actionType] = reducersByActionType[actionType][0];
    }
  };

  for (var actionType in reducersByActionType) {
    _loop(actionType);
  }

  return reducer;
}

// ACKNOWLEDGE
// Place record of request on store
function acknowledgeReducer(state, _ref) {
  var request = _ref.request;

  return (0, _lodash2.default)({}, state, {
    __nextQueryId: request.id + 1,
    queries: _defineProperty({}, request.id, {
      id: request.id,
      prepared: (0, _isNodeFn2.default)(),
      complete: false,
      OK: null
    })
  });
}

// COMPLETE
// Place entity on the store; update query record if for component (has an id)
function completeReducer(normalise) {
  return function (state, _ref2) {
    var request = _ref2.request;
    var idAttribute = state.__keyEntitiesBy;

    var query = state.queries[request.id];
    var newState = (0, _lodash2.default)({}, state);

    if (request.id !== _constants.PreloadQueryId && !query) {
      console.log('[kasia] attempt to complete non-existent query. Ignoring. Request:', { request: request });
      return newState;
    }

    newState.entities = (0, _lodash2.default)(newState.entities, normalise(request.result, { idAttribute: idAttribute }));

    if (request.id === _constants.PreloadQueryId) {
      return newState;
    }

    var newQuery = {
      id: request.id,
      prepared: (0, _isNodeFn2.default)(),
      complete: true,
      OK: true
    };

    if (query.preserve) {
      newQuery.result = request.result;
      newQuery.paging = null;
    } else {
      newQuery.entities = (0, _pickEntityIds2.default)(request.result);
      newQuery.paging = request.result._paging || {};
    }

    newState.queries[request.id] = newQuery;

    return newState;
  };
}

// FAIL
// Update query record only
function failReducer(state, _ref3) {
  var request = _ref3.request;

  return (0, _lodash2.default)({}, state, {
    queries: _defineProperty({}, request.id, {
      id: request.id,
      error: request.error,
      prepared: (0, _isNodeFn2.default)(),
      complete: true,
      OK: false
    })
  });
}

function rewindReducer(state) {
  return (0, _lodash2.default)({}, state, {
    __nextQueryId: 0
  });
}

function incrementNextQueryIdReducer(state) {
  return (0, _lodash2.default)({}, state, {
    __nextQueryId: state.__nextQueryId + 1
  });
}

/**
 * Create the aggregate reducer for an instance of Kasia.
 * @param {Object} keyEntitiesBy Entity property used as key in store
 * @param {Object} reducers Plugin reducers
 * @returns {Object} Kasia reducer
 */
function createReducer(_ref4) {
  var keyEntitiesBy = _ref4.keyEntitiesBy,
      reducers = _ref4.reducers;

  var normaliser = function normaliser(data) {
    return (0, _normalise2.default)(data, { idAttribute: keyEntitiesBy });
  };
  var reducer = mergeNativeAndThirdPartyReducers(reducers, normaliser);

  var initialState = Object.assign({}, INITIAL_STATE, { __keyEntitiesBy: keyEntitiesBy });

  return {
    wordpress: function kasiaReducer() {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
      var action = arguments[1];

      var _action$type$split = action.type.split('/'),
          _action$type$split2 = _slicedToArray(_action$type$split, 1),
          actionNamespace = _action$type$split2[0];

      if (actionNamespace === 'kasia' && action.type in reducer) {
        return reducer[action.type](state, action);
      }

      return state;
    }
  };
}