'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._getCurrentQueryId = _getCurrentQueryId;
exports.fetch = fetch;
exports.watchRequests = watchRequests;

var _effects = require('redux-saga/effects');

var _wpapi = require('../wpapi');

var _wpapi2 = _interopRequireDefault(_wpapi);

var _constants = require('../constants');

var _actions = require('./actions');

var _queryBuilder = require('../util/query-builder');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _marked = /*#__PURE__*/regeneratorRuntime.mark(fetch),
    _marked2 = /*#__PURE__*/regeneratorRuntime.mark(watchRequests);

function _getCurrentQueryId(state) {
  return state.wordpress.__nextQueryId;
}

/**
 * Make a fetch request to the WP-API according to the action
 * object and record the result in the store.
 * @param {Object} action Action object
 */
function fetch(_ref) {
  var request = _ref.request;
  var wpapi, fn, data;
  return regeneratorRuntime.wrap(function fetch$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.prev = 0;
          _context.next = 3;
          return (0, _effects.select)(_getCurrentQueryId);

        case 3:
          request.id = _context.sent;
          _context.next = 6;
          return (0, _effects.put)((0, _actions.acknowledgeRequest)(request));

        case 6:
          wpapi = (0, _wpapi2.default)();
          fn = request.queryFn || (0, _queryBuilder.buildQueryFunction)(request);
          _context.next = 10;
          return (0, _effects.call)(fn, wpapi);

        case 10:
          data = _context.sent;
          _context.next = 13;
          return (0, _effects.put)((0, _actions.completeRequest)(request.id, data));

        case 13:
          _context.next = 19;
          break;

        case 15:
          _context.prev = 15;
          _context.t0 = _context['catch'](0);
          _context.next = 19;
          return (0, _effects.put)((0, _actions.failRequest)(request.id, _context.t0.stack || _context.t0.message));

        case 19:
        case 'end':
          return _context.stop();
      }
    }
  }, _marked, this, [[0, 15]]);
}

/** Watch request create actions and fetch data for them. */
function watchRequests() {
  return regeneratorRuntime.wrap(function watchRequests$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _context2.next = 2;
          return (0, _effects.takeEvery)([_constants.ActionTypes.RequestCreatePost, _constants.ActionTypes.RequestCreateQuery], fetch);

        case 2:
        case 'end':
          return _context2.stop();
      }
    }
  }, _marked2, this);
}