'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _filters = require('wpapi/lib/mixins/filters');

var _filters2 = _interopRequireDefault(_filters);

var _parameters = require('wpapi/lib/mixins/parameters');

var _parameters2 = _interopRequireDefault(_parameters);

var _humps = require('humps');

var _humps2 = _interopRequireDefault(_humps);

var _wpapi = require('../wpapi');

var _wpapi2 = _interopRequireDefault(_wpapi);

var _invariants = require('../invariants');

var _invariants2 = _interopRequireDefault(_invariants);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/** Assert that an object has all `keys`. */
function hasKeys(obj) {
  for (var _len = arguments.length, keys = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    keys[_key - 1] = arguments[_key];
  }

  return keys.reduce(function (bool, key) {
    if (!bool) return bool;
    return obj.hasOwnProperty(key);
  }, true);
}

var optionsCache = new Map();

var contentTypes = { register: register, get: get, getAll: getAll, derive: derive };

exports.default = contentTypes;

// Pre-populate cache with built-in content type options.

Object.keys(_constants.ContentTypes).forEach(function (key) {
  var name = _constants.ContentTypes[key];
  var plural = _constants.ContentTypesPlural[name];
  var slug = _constants.ContentTypesPlural[name];
  contentTypes.register({ name: name, plural: plural, slug: slug }, true);
});

/** Create and set options object for a type in the cache and register on wpapi instance. */
function register(contentType, builtIn) {
  _invariants2.default.isValidContentTypeObject(contentType);
  _invariants2.default.isNewContentType(contentTypes.getAll(), contentType);

  var _contentType$namespac = contentType.namespace,
      namespace = _contentType$namespac === undefined ? _constants.WpApiNamespace : _contentType$namespac,
      name = contentType.name,
      plural = contentType.plural,
      slug = contentType.slug,
      _route = contentType.route,
      _methodName = contentType.methodName;


  var route = _route || '/' + slug + '/(?P<id>)';
  var methodName = _humps2.default.camelize(_methodName || plural);
  var mixins = Object.assign({}, _filters2.default, _parameters2.default);
  var options = Object.assign({}, contentType, { route: route, methodName: methodName });

  // Only register custom types with node-wpapi instance as built-ins are already available
  if (!builtIn) {
    var WP = (0, _wpapi2.default)();
    WP[methodName] = WP.registerRoute(namespace, route, { mixins: mixins });
  }

  optionsCache.set(name, options);
}

/** Get the options for a content type. */
function get(contentType) {
  return optionsCache.get(contentType);
}

/** Get all registered content types and their options. */
function getAll() {
  return optionsCache;
}

/** Derive the content type of an entity from the WP-API. */
function derive(entity) {
  if (!entity) {
    throw new Error('Expecting entity to be an object, got "' + (typeof entity === 'undefined' ? 'undefined' : _typeof(entity)) + '".');
  }

  if (typeof entity.type !== 'undefined') {
    switch (entity.type) {
      case 'attachment':
        return _constants.ContentTypes.Media;
      case 'comment':
        return _constants.ContentTypes.Comment;
      case 'page':
        return _constants.ContentTypes.Page;
      case 'post':
        return _constants.ContentTypes.Post;
      default:
        return entity.type; // Custom content type
    }
  }

  if (typeof entity.taxonomy !== 'undefined') {
    if (entity.taxonomy === 'post_tag') return _constants.ContentTypes.Tag;
    if (entity.taxonomy === 'category') return _constants.ContentTypes.Category;
    return _constants.ContentTypes.Taxonomy;
  }

  if (entity.avatar_urls) return _constants.ContentTypes.User;
  if (Array.isArray(entity.types)) return _constants.ContentTypes.Taxonomy;
  if (hasKeys(entity, 'public', 'queryable', 'slug')) return _constants.ContentTypes.PostStatus;
  if (hasKeys(entity, 'description', 'hierarchical', 'name')) return _constants.ContentTypes.PostType;

  return null;
}