'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = findEntities;
/** Filter `entities` to contain only those whose `keyToInspect` is in `identifiers`. */
function findEntities(entities, identifiers, keyToInspect) {
  identifiers = identifiers.map(String);

  var reduced = {};

  for (var entityTypeName in entities) {
    var entitiesOfType = entities[entityTypeName];

    for (var key in entitiesOfType) {
      var entity = entitiesOfType[key];

      // Try to find entity by `keyToInspect` but fall back on id and then slug as
      // for entities that don't have an `id` identifiers will contain their slug
      // and vice-versa for entities that don't have a `slug`
      var entityId = void 0;

      if (typeof entity[keyToInspect] !== 'undefined') {
        entityId = entity[keyToInspect];
      } else if (typeof entity.id !== 'undefined') {
        entityId = entity.id;
      } else {
        entityId = entity.slug;
      }

      entityId = String(entityId);

      if (identifiers.indexOf(entityId) !== -1) {
        reduced[entityTypeName] = reduced[entityTypeName] || {};
        reduced[entityTypeName][key] = entity;
      }
    }
  }

  return reduced;
}