'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = normalise;

var _normalizr = require('normalizr');

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _castarray = require('castarray');

var _castarray2 = _interopRequireDefault(_castarray);

var _schemasManager = require('./schemas-manager');

var _schemasManager2 = _interopRequireDefault(_schemasManager);

var _contentTypesManager = require('./content-types-manager');

var _contentTypesManager2 = _interopRequireDefault(_contentTypesManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/** Split a response from the WP-API into its constituent entities. */
function normalise(response, _ref) {
  var idAttribute = _ref.idAttribute;

  var schemas = _schemasManager2.default.getAll() || _schemasManager2.default.init(idAttribute);

  return (0, _castarray2.default)(response).reduce(function (entities, entity) {
    var type = _contentTypesManager2.default.derive(entity);

    if (!type) {
      console.log('[kasia] could not derive entity type - ignoring.', 'Entity: ' + (entity ? JSON.stringify(entity) : typeof entity === 'undefined' ? 'undefined' : _typeof(entity)));
      return entities;
    }

    var contentTypeSchema = schemas[type]
    // Built-in content type or previously registered custom content type
    ? schemas[type]
    // Custom content type, will only get here once for each type
    : _schemasManager2.default.createSchema(type, idAttribute);

    var schema = Array.isArray(entity) ? (0, _normalizr.arrayOf)(contentTypeSchema) : contentTypeSchema;

    var normalised = (0, _normalizr.normalize)(entity, schema);

    return (0, _lodash2.default)(entities, normalised.entities);
  }, {});
}