'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = pickEntityIds;

var _pickToArray = require('pick-to-array');

var _pickToArray2 = _interopRequireDefault(_pickToArray);

var _contentTypesManager = require('./content-types-manager');

var _contentTypesManager2 = _interopRequireDefault(_contentTypesManager);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * Pick all entity identifiers from a WP-API response.
 * @param {Object} data Raw WP-API JSON
 * @returns {Array} Entity identifiers
 */
function pickEntityIds(data) {
  var entityIdentifiers = (0, _pickToArray2.default)(data, 'id');

  // Accommodate content types that do not have an `id` property
  for (var i = 0, len = data.length; i < len; i++) {
    var entity = data[i];
    var type = _contentTypesManager2.default.derive(entity);
    if (_constants.ContentTypesWithoutId.includes(type)) {
      entityIdentifiers.push.apply(entityIdentifiers, _toConsumableArray((0, _pickToArray2.default)(entity, 'slug')));
    }
  }

  return entityIdentifiers;
}