'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deriveQueryFunction = deriveQueryFunction;
exports.buildQueryFunction = buildQueryFunction;

var _contentTypesManager = require('./content-types-manager');

var _contentTypesManager2 = _interopRequireDefault(_contentTypesManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/** Fetch data for a single post via the `wpapi` instance. */
function queryFn(wpapi, contentTypeMethodName, idTypeMethodName, id) {
  var contentTypeApi = wpapi[contentTypeMethodName]();
  var query = contentTypeApi[idTypeMethodName](id);
  return query.embed().get();
}

/**
 * Create a function that dynamically calls the necessary wpapi
 * methods that will fetch data for the given content type item
 * and given identifier (ID or slug depending on its type).
 *
 * @example
 * ```js
 * deriveQueryFunction("posts", 16)
 * //=> () => WP.posts().id(16).embed().get()
 * ```
 *
 * @param {Object} contentTypeMethodName The method name on wpapi instance
 * @param {String|Number} identifier The identifier's id or slug
 * @returns {Function} A function to make a request to the WP-API
 */
function deriveQueryFunction(contentTypeMethodName, identifier) {
  var idMethodName = typeof identifier === 'string' ? 'slug' : 'id';
  return function (wpapi) {
    return queryFn(wpapi, contentTypeMethodName, idMethodName, identifier);
  };
}

/** Given an `action` produce a function that will query the WP-API. */
function buildQueryFunction(request) {
  var options = _contentTypesManager2.default.get(request.contentType);
  return deriveQueryFunction(options.methodName, request.identifier);
}