'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = runSagas;
/**
 * Run all `sagas` until they are complete.
 * @param {Object} store Enhanced redux store with `runSaga` method
 * @param {Array} sagas Array of saga operations
 * @returns {Promise}
 */
function runSagas(store, sagas) {
  if ((typeof store === 'undefined' ? 'undefined' : _typeof(store)) !== 'object' || typeof store.runSaga !== 'function') {
    throw new Error('Expecting store to be redux store with runSaga enhancer method.');
  }

  return sagas.reduce(function (promise, getSaga) {
    return promise.then(function () {
      var state = store.getState();
      return store.runSaga(getSaga(state)).done;
    });
  }, Promise.resolve());
}