'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _normalizr = require('normalizr');

var _constants = require('../constants');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var schemasManager = { __flush__: __flush__, getAll: getAll, createSchema: createSchema, init: init };

exports.default = schemasManager;

/** Schema object cache, populated in `makeSchemas`. */

var schemas = void 0;

/** Individual schema definitions, defined like this so we can reference one from another. */
var categorySchema = void 0,
    mediaSchema = void 0,
    pageSchema = void 0,
    postSchema = void 0,
    revisionSchema = void 0,
    tagSchema = void 0,
    userSchema = void 0,
    commentSchema = void 0,
    postTypeSchema = void 0,
    postStatusSchema = void 0,
    taxonomySchema = void 0;

/** Invalidate the schema object cache. */
function __flush__() {
  schemas = null;
}

/** Get the schema object cache. */
function getAll() {
  return schemas;
}

/** Create a custom schema definition (for custom content types). */
function createSchema(name, idAttribute) {
  if (!schemas) {
    throw new Error('createSchema called before schema cache populated, call makeSchemas first.');
  } else if (typeof name !== 'string') {
    throw new Error('Expecting name to be a string, got "' + (typeof name === 'undefined' ? 'undefined' : _typeof(name)) + '".');
  } else if (typeof idAttribute !== 'string') {
    throw new Error('Expecting idAttribute to be a string, got "' + (typeof idAttribute === 'undefined' ? 'undefined' : _typeof(idAttribute)) + '".');
  }

  var contentTypeSchema = new _normalizr.Schema(name, { idAttribute: idAttribute });

  contentTypeSchema.define({
    author: userSchema,
    post: postSchema,
    featuredMedia: mediaSchema
  });

  return contentTypeSchema;
}

/** Populate the cache of schemas for built-in content types. */
function init(idAttribute) {
  var _schemas;

  if (typeof idAttribute !== 'string') {
    throw new Error('Expecting idAttribute to be a string, got "' + (typeof idAttribute === 'undefined' ? 'undefined' : _typeof(idAttribute)) + '".');
  }

  if (schemas) return schemas;

  // Content types with `id` properties
  categorySchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Category], { idAttribute: idAttribute });
  commentSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Comment], { idAttribute: idAttribute });
  mediaSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Media], { idAttribute: idAttribute });
  pageSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Page], { idAttribute: idAttribute });
  postSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Post], { idAttribute: idAttribute });
  revisionSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.PostRevision], { idAttribute: idAttribute });
  tagSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Tag], { idAttribute: idAttribute });
  userSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.User], { idAttribute: idAttribute });

  // Content types without `id` properties
  postTypeSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.PostType], { idAttribute: 'slug' });
  postStatusSchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.PostStatus], { idAttribute: 'slug' });
  taxonomySchema = new _normalizr.Schema(_constants.ContentTypesPlural[_constants.ContentTypes.Taxonomy], { idAttribute: 'slug' });

  mediaSchema.define({
    author: userSchema,
    post: postSchema
  });

  pageSchema.define({
    author: userSchema,
    featuredMedia: mediaSchema
  });

  postSchema.define({
    author: userSchema,
    featuredMedia: mediaSchema,
    categories: (0, _normalizr.arrayOf)(categorySchema),
    tags: (0, _normalizr.arrayOf)(tagSchema)
  });

  schemas = (_schemas = {}, _defineProperty(_schemas, _constants.ContentTypes.Category, categorySchema), _defineProperty(_schemas, _constants.ContentTypes.Comment, commentSchema), _defineProperty(_schemas, _constants.ContentTypes.Media, mediaSchema), _defineProperty(_schemas, _constants.ContentTypes.Page, pageSchema), _defineProperty(_schemas, _constants.ContentTypes.Post, postSchema), _defineProperty(_schemas, _constants.ContentTypes.PostStatus, postStatusSchema), _defineProperty(_schemas, _constants.ContentTypes.PostType, postTypeSchema), _defineProperty(_schemas, _constants.ContentTypes.PostRevision, revisionSchema), _defineProperty(_schemas, _constants.ContentTypes.Tag, tagSchema), _defineProperty(_schemas, _constants.ContentTypes.Taxonomy, taxonomySchema), _defineProperty(_schemas, _constants.ContentTypes.User, userSchema), _schemas);

  return schemas;
}